import React, { useState, useEffect, useRef } from 'react'
import { AccountService } from 'services/accountService'

import { FluidType } from 'enum/fluid.enum'

import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import { getPicto, getAddPicto, getFuildType, getParamPicto } from 'utils/utils'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import chevronUp from 'assets/icons/ico/chevron-up.svg'

import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'

import failurePicto from 'assets/png/picto/picto-failure.png'
import IFluidConfig from 'services/IFluidConfig'

import KonnectorNotFound from 'components/ContentComponents/Konnector/KonnectorNotFound'
import KonnectorForm from 'components/ContentComponents/Konnector/KonnectorForm'
import KonnectorResult from 'components/ContentComponents/Konnector/KonnectorResult'
import KonnectorLaunch from 'components/ContentComponents/Konnector/KonnectorLaunch'

import { Konnector, Trigger, TriggerState } from 'doctypes'
import { JobState } from 'services/jobsService'
import { TriggerService } from 'services/triggersService'

interface KonnectorViewerCardProps {
  fluidConfig: IFluidConfig
  konnector: Konnector
  client: Client
  isParam: boolean
  t: Function
}

const KonnectorViewerCard: React.FC<KonnectorViewerCardProps> = ({
  fluidConfig,
  konnector,
  client,
  isParam,
  t,
}: KonnectorViewerCardProps) => {
  const [account, setAccount] = useState<Account | null>(null)
  const [trigger, setTrigger] = useState<Trigger | null>(null)
  const [triggerState, setTriggerState] = useState<TriggerState | null>(null)
  const [shouldLaunch, setLaunch] = useState<boolean>(false)

  const [setActive, setActiveState] = useState('')
  const [setHeight, setHeightState] = useState('0px')

  const [jobState, setJobState] = useState<string>('')

  const [isLoading, setLoading] = useState<boolean>(true)

  const content: React.MutableRefObject<null> = useRef<null>(null)
  const type: string = fluidConfig.konnectorConfig.type
  const fluid: FluidType = getFuildType(type)
  const iconType = getPicto(fluid)

  const iconAddType = isParam ? getParamPicto(fluid) : getAddPicto(fluid)
  const loginFailed: boolean =
    triggerState != null &&
    triggerState.last_error != undefined &&
    triggerState.last_error === 'LOGIN_FAILED' &&
    triggerState.status != undefined &&
    triggerState.status === 'errored'

  const toggleAccordion = () => {
    setActiveState(setActive === '' ? 'active' : '')
    setHeightState(
      setActive === 'active' &&
        content &&
        content.current &&
        content.current.scrollHeight
        ? '0px'
        : `${content.current.scrollHeight}px`
    )
  }

  const getKonnectorStateMarkup = () => {
    return jobState === JobState.Errored ? (
      <img className="state-icon" src={failurePicto}></img>
    ) : (
      ''
    )
  }

  const updateState = async (trigger: Trigger) => {
    const triggerState = await TriggerService.fetchTriggerState(client, trigger)
    if (triggerState) {
      setTriggerState(triggerState)
    }
  }

  const handleSuccessForm = (_account: Account, _trigger: Trigger) => {
    setAccount(_account)
    setTrigger(_trigger)
    setLaunch(true)
  }

  const handleKonnectorLaunch = async () => {
    if (trigger) {
      await updateState(trigger)
    }
    setLaunch(false)
  }

  const handleJobState = (_jobState: JobState) => {
    setJobState(_jobState)
  }

  useEffect(() => {
    let subscribed = true
    async function getData() {
      setLoading(true)
      if (konnector) {
        const _account = await AccountService.getAccountByType(
          client,
          konnector.slug
        )
        if (subscribed && _account) {
          setAccount(_account)
          const _trigger = await TriggerService.fetchTriggerFromAccount(
            client,
            _account
          )
          if (subscribed && _trigger) {
            setTrigger(_trigger)
            await updateState(_trigger)
          }
        }
      }
      setLoading(false)
    }
    getData()
    return () => {
      subscribed = false
    }
  }, [])

  return (
    <>
      {isLoading ? null : (
        <div className={`accordion ${setActive}`}>
          <div>
            <div
              className={`accordion-header ${setActive}`}
              onClick={toggleAccordion}
            >
              <div className="accordion-icon">
                {account && !loginFailed ? (
                  <StyledIcon className="icon" icon={iconType} size={49} />
                ) : (
                  <StyledIcon className="icon" icon={iconAddType} size={49} />
                )}
              </div>
              <div className="state-picto">{getKonnectorStateMarkup()}</div>
              <div className="accordion-info">
                <div className="accordion-title text-16-normal">
                  {account && !loginFailed
                    ? t('FLUID.' + FluidType[fluid] + '.LABEL')
                    : t('KONNECTORCONFIG.LABEL_CONNECTTO_' + FluidType[fluid])}
                </div>
              </div>
              <StyledIconButton icon={setActive ? chevronUp : chevronDown} />
            </div>

            <div
              ref={content}
              style={{ maxHeight: `${setHeight}` }}
              className={`accordion-content ${setActive}`}
            >
              {!konnector ? (
                <KonnectorNotFound
                  konnectorSlug={fluidConfig.konnectorConfig.slug}
                />
              ) : shouldLaunch && trigger ? (
                <KonnectorLaunch
                  trigger={trigger}
                  type={type}
                  handleKonnectorLaunch={handleKonnectorLaunch}
                />
              ) : account && !loginFailed ? (
                <KonnectorResult
                  account={account}
                  handleJobState={handleJobState}
                />
              ) : (
                <KonnectorForm
                  fluidConfig={fluidConfig}
                  konnector={konnector}
                  account={account}
                  trigger={trigger}
                  handleSuccessForm={handleSuccessForm}
                />
              )}
            </div>
          </div>
        </div>
      )}
    </>
  )
}
export default translate()(withClient(KonnectorViewerCard))
